// This file was auto-generated from a YAML file

package v1_0

func init() {
	Profile["/hot/1.0/js/functions/and.js"] = `

// [https://docs.openstack.org/heat/wallaby/template_guide/hot_spec.html#and]

exports.evaluate = function() {
	return 'TODO';
};`
}
