// This file was auto-generated from a YAML file

package v5_0_5

func init() {
	Profile["/cloudify/5.0.5/js/functions/get_capability.js"] = `

// [https://docs.cloudify.co/5.0.5/developer/blueprints/spec-intrinsic-functions/]

exports.evaluate = function() {
	return 'TODO';
};
`
}
