// This file was auto-generated from a YAML file

package v1_0

func init() {
	Profile["/hot/1.0/js/constraints/custom_constraint.js"] = `

exports.validate = function() {
	// TODO
	return true;
};
`
}
